@@ -1,8 +1,7 @@
 # Author: Ingo Feinerer
 
-getSources <- function() {
-   c("CSVSource", "DirSource", "GmaneSource", "ReutersSource", "VectorSource")
-}
+getSources <- function()
+   c("CSVSource", "DataframeSource", "DirSource", "GmaneSource", "ReutersSource", "URISource", "VectorSource")
 
 # Source objects
 
@@ -19,6 +18,16 @@ setClass("VectorSource",
          representation(Content = "vector"),
          contains = c("Source"))
 
+# A data frame where each row is interpreted as document
+setClass("DataframeSource",
+         representation(Content = "data.frame"),
+         contains = c("Source"))
+
+# A single document identified by a Uniform Resource Identifier
+setClass("URISource",
+         representation(URI = "call"),
+         contains = c("Source"))
+
 # A directory with files
 setClass("DirSource",
          representation(FileList = "character"),
@@ -26,20 +35,20 @@ setClass("DirSource",
 
 # A single CSV file where each line is interpreted as document
 setClass("CSVSource",
-         representation(URI = "ANY",
+         representation(URI = "call",
                         Content = "character"),
          contains = c("Source"))
 
 # A single XML file consisting of several Reuters documents
 # Works both for Reuters21578XML and RCV1 XML files
 setClass("ReutersSource",
-         representation(URI = "ANY",
+         representation(URI = "call",
                         Content = "list"),
          contains = c("Source"))
 
 # A single XML (RDF) file containing Gmane mailing list archive feeds
 setClass("GmaneSource",
-         representation(URI = "ANY",
+         representation(URI = "call",
                         Content = "list"),
          contains = c("Source"))
 
@@ -54,6 +63,10 @@ setMethod("VectorSource",
                   DefaultReader = readPlain, Encoding = encoding, Length = length(object))
           })
 
+DataframeSource<- function(object, encoding = "UTF-8")
+    new("DataframeSource", LoDSupport = FALSE, Content = object, Position = 0,
+        DefaultReader = readPlain, Encoding = encoding, Length = nrow(object))
+
 setGeneric("DirSource", function(directory, encoding = "UTF-8", recursive = FALSE) standardGeneric("DirSource"))
 setMethod("DirSource",
           signature(directory = "character"),
@@ -65,24 +78,30 @@ setMethod("DirSource",
                   Position = 0, DefaultReader = readPlain, Encoding = encoding, Length = length(files))
           })
 
+setGeneric("URISource", function(object, encoding = "UTF-8") standardGeneric("URISource"))
+setMethod("URISource", signature(object = "character"),
+          function(object, encoding = "UTF-8")
+              new("URISource", LoDSupport = TRUE, URI = substitute(file(object, encoding = encoding)),
+                  Position = 0, DefaultReader = readPlain, Encoding = encoding, Length = 1))
+setMethod("URISource", signature(object = "connection"),
+          function(object, encoding = "UTF-8")
+              new("URISource", LoDSupport = TRUE, URI = match.call()$object,
+                  Position = 0, DefaultReader = readPlain, Encoding = encoding, Length = 1))
+
 setGeneric("CSVSource", function(object, encoding = "UTF-8") standardGeneric("CSVSource"))
 setMethod("CSVSource",
           signature(object = "character"),
           function(object, encoding = "UTF-8") {
-              object <- substitute(file(object, encoding = encoding))
-              con <- eval(object)
-              content <- apply(read.csv(con), 1, paste, collapse = " ")
-              new("CSVSource", LoDSupport = FALSE, URI = object,
+              content <- apply(read.csv(object, encoding = encoding), 1, paste, collapse = " ")
+              new("CSVSource", LoDSupport = FALSE, URI = substitute(file(object, encoding = encoding)),
                   Content = content, Position = 0, DefaultReader = readPlain,
                   Encoding = encoding, Length = length(content))
           })
 setMethod("CSVSource",
-          signature(object = "ANY"),
+          signature(object = "connection"),
           function(object, encoding = "UTF-8") {
-              object <- substitute(object)
-              con <- eval(object)
-              content <- apply(read.csv(con), 1, paste, collapse = " ")
-              new("CSVSource", LoDSupport = FALSE, URI = object,
+              content <- apply(read.csv(object), 1, paste, collapse = " ")
+              new("CSVSource", LoDSupport = FALSE, URI = match.call()$object,
                   Content = content, Position = 0, DefaultReader = readPlain,
                   Encoding = encoding, Length = length(content))
           })
@@ -91,28 +110,22 @@ setGeneric("ReutersSource", function(object, encoding = "UTF-8") standardGeneric
 setMethod("ReutersSource",
           signature(object = "character"),
           function(object, encoding = "UTF-8") {
-              object <- substitute(file(object, encoding = encoding))
-              con <- eval(object)
-              corpus <- paste(readLines(con), "\n", collapse = "")
-              close(con)
+              corpus <- paste(readLines(object, encoding = encoding), "\n", collapse = "")
               tree <- xmlTreeParse(corpus, asText = TRUE)
               content <- xmlRoot(tree)$children
 
-              new("ReutersSource", LoDSupport = FALSE, URI = object,
+              new("ReutersSource", LoDSupport = FALSE, URI = substitute(file(object, encoding = encoding)),
                   Content = content, Position = 0, DefaultReader = readReut21578XML,
                   Encoding = encoding, Length = length(content))
           })
 setMethod("ReutersSource",
-          signature(object = "ANY"),
+          signature(object = "connection"),
           function(object, encoding = "UTF-8") {
-              object <- substitute(object)
-              con <- eval(object)
-              corpus <- paste(readLines(con), "\n", collapse = "")
-              close(con)
+              corpus <- paste(readLines(object), "\n", collapse = "")
               tree <- xmlTreeParse(corpus, asText = TRUE)
               content <- xmlRoot(tree)$children
 
-              new("ReutersSource", LoDSupport = FALSE, URI = object,
+              new("ReutersSource", LoDSupport = FALSE, URI = match.call()$object,
                   Content = content, Position = 0, DefaultReader = readReut21578XML,
                   Encoding = encoding, Length = length(content))
           })
@@ -121,73 +134,40 @@ setGeneric("GmaneSource", function(object, encoding = "UTF-8") standardGeneric("
 setMethod("GmaneSource",
           signature(object = "character"),
           function(object, encoding = "UTF-8") {
-              object <- substitute(file(object, encoding = encoding))
-              con <- eval(object)
-              corpus <- paste(readLines(con), "\n", collapse = "")
-              close(con)
+              corpus <- paste(readLines(object, encoding = encoding), "\n", collapse = "")
               tree <- xmlTreeParse(corpus, asText = TRUE)
               content <- xmlRoot(tree)$children
               content <- content[names(content) == "item"]
 
-              new("GmaneSource", LoDSupport = FALSE, URI = object,
+              new("GmaneSource", LoDSupport = FALSE, URI = substitute(file(object, encoding = encoding)),
                   Content = content, Position = 0, DefaultReader = readGmane,
                   Encoding = encoding, Length = length(content))
           })
 setMethod("GmaneSource",
-          signature(object = "ANY"),
+          signature(object = "connection"),
           function(object, encoding = "UTF-8") {
-              object <- substitute(object)
-              con <- eval(object)
-              corpus <- paste(readLines(con), "\n", collapse = "")
-              close(con)
+              corpus <- paste(readLines(object), "\n", collapse = "")
               tree <- xmlTreeParse(corpus, asText = TRUE)
               content <- xmlRoot(tree)$children
               content <- content[names(content) == "item"]
 
-              new("GmaneSource", LoDSupport = FALSE, URI = object,
+              new("GmaneSource", LoDSupport = FALSE, URI = match.call()$object,
                   Content = content, Position = 0, DefaultReader = readGmane,
                   Encoding = encoding, Length = length(content))
           })
 
 setGeneric("stepNext", function(object) standardGeneric("stepNext"))
-setMethod("stepNext",
-          signature(object = "VectorSource"),
-          function(object) {
-              object@Position <- object@Position + 1
-              object
-          })
-setMethod("stepNext",
-          signature(object = "DirSource"),
-          function(object) {
-              object@Position <- object@Position + 1
-              object
-          })
-setMethod("stepNext",
-          signature(object = "CSVSource"),
-          function(object) {
-              object@Position <- object@Position + 1
-              object
-          })
-setMethod("stepNext",
-          signature(object = "ReutersSource"),
-          function(object) {
-              object@Position <- object@Position + 1
-              object
-          })
-setMethod("stepNext",
-          signature(object = "GmaneSource"),
+setMethod("stepNext", signature(object = "Source"),
           function(object) {
               object@Position <- object@Position + 1
               object
           })
 
 setGeneric("getElem", function(object) standardGeneric("getElem"))
-setMethod("getElem",
-          signature(object = "VectorSource"),
-          function(object) {
-              list(content = object@Content[object@Position],
-                   uri = NULL)
-          })
+setMethod("getElem", signature(object = "VectorSource"),
+          function(object) list(content = object@Content[object@Position], uri = NULL))
+setMethod("getElem", signature(object = "DataframeSource"),
+          function(object) list(content = object@Content[object@Position, ], uri = NULL))
 setMethod("getElem",
           signature(object = "DirSource"),
           function(object) {
@@ -196,6 +176,8 @@ setMethod("getElem",
               list(content = readLines(filename, encoding = encoding),
                    uri = substitute(file(filename, encoding = encoding)))
           })
+setMethod("getElem", signature(object = "URISource"),
+          function(object) list(content = readLines(eval(object@URI)), uri = object@URI))
 setMethod("getElem",
           signature(object = "CSVSource"),
           function(object) {
@@ -226,43 +208,17 @@ setMethod("getElem",
           })
 
 setGeneric("eoi", function(object) standardGeneric("eoi"))
-setMethod("eoi",
-          signature(object = "VectorSource"),
-          function(object) {
-              if (length(object@Content) <= object@Position)
-                  return(TRUE)
-              else
-                  return(FALSE)
-          })
-setMethod("eoi",
-          signature(object = "DirSource"),
-          function(object) {
-              if (length(object@FileList) <= object@Position)
-                  return(TRUE)
-              else
-                  return(FALSE)
-          })
-setMethod("eoi",
-          signature(object = "CSVSource"),
-          function(object) {
-              if (length(object@Content) <= object@Position)
-                  return(TRUE)
-              else
-                  return(FALSE)
-          })
-setMethod("eoi",
-          signature(object = "ReutersSource"),
-          function(object) {
-              if (length(object@Content) <= object@Position)
-                  return(TRUE)
-              else
-                  return(FALSE)
-          })
-setMethod("eoi",
-          signature(object = "GmaneSource"),
-          function(object) {
-              if (length(object@Content) <= object@Position)
-                  return(TRUE)
-              else
-                  return(FALSE)
-          })
+setMethod("eoi", signature(object = "VectorSource"),
+          function(object) return(length(object@Content) <= object@Position))
+setMethod("eoi", signature(object = "DataframeSource"),
+          function(object) return(nrow(object@Content) <= object@Position))
+setMethod("eoi", signature(object = "DirSource"),
+          function(object) return(length(object@FileList) <= object@Position))
+setMethod("eoi", signature(object = "URISource"),
+          function(object) return(1 <= object@Position))
+setMethod("eoi", signature(object = "CSVSource"),
+          function(object) return(length(object@Content) <= object@Position))
+setMethod("eoi", signature(object = "ReutersSource"),
+          function(object) return(length(object@Content) <= object@Position))
+setMethod("eoi", signature(object = "GmaneSource"),
+          function(object) return(length(object@Content) <= object@Position))
