@@ -152,7 +152,7 @@ setMethod("tmUpdate",
               if (is.null(readerControl$load))
                   readerControl$load = TRUE
 
-              object.filelist <- unlist(lapply(object, function(x) {as.character(URI(x))[2]}))
+              object.filelist <- unlist(lapply(object, function(x) {summary(eval(URI(x)))$description}))
               new.files <- setdiff(origin@FileList, object.filelist)
 
               for (filename in new.files) {
@@ -266,7 +266,7 @@ setMethod("asPlain",
 setMethod("asPlain",
           signature(object = "NewsgroupDocument"),
           function(object, FUN, ...) {
-              new("PlainTextDocument", .Data = Content(object), Cached = TRUE, URI = "", Author = Author(object),
+              new("PlainTextDocument", .Data = Content(object), Cached = TRUE, URI = NULL, Author = Author(object),
                   DateTimeStamp = DateTimeStamp(object), Description = Description(object), ID = ID(object),
                   Origin = Origin(object), Heading = Heading(object), Language = Language(object),
                   LocalMetaData = LocalMetaData(object))
@@ -275,7 +275,7 @@ setMethod("asPlain",
           signature(object = "StructuredTextDocument"),
           function(object, FUN, ...) {
               new("PlainTextDocument", .Data = unlist(Content(object)), Cached = TRUE,
-                  URI = "", Author = Author(object), DateTimeStamp = DateTimeStamp(object),
+                  URI = NULL, Author = Author(object), DateTimeStamp = DateTimeStamp(object),
                   Description = Description(object), ID = ID(object), Origin = Origin(object),
                   Heading = Heading(object), Language = Language(object),
                   LocalMetaData = LocalMetaData(object))
